{ABC Notation to Parallax LCD with pizospeaker}

CON
        _clkmode = xtal1 + pll16x    'Standard clock mode 
        _xinfreq = 5_000_000         '* crystal frequency = 80 MHz
        CR = 13, EOF = $FF
        #220, An,ASn,Bn,Cn,CSn,Dn,DSn,En,Fn,FSn,Gn,GSn  'values for notes
        #215, Scale3, Scale4, Scale5, Scale6, Scale7    'values for scales
        #208, Sixtyfourth, ThirtySecond, Sixteenth, Eighth, Quarter, Half, Whole
        
        ABit = $80, BBit = $40, CBit = $20, DBit = $10, EBit = $08, FBit = $04, GBit = $02
  
VAR
  long  myascii, mypoint          'used to access file
  byte  key, flats, sharps        'the key we are in and corresponding flats and sharps
  byte  havekey                   '1 -> we have found the key
  byte  HeaderKey                 'the first char of each header field
  byte  LCode                       'basic note length code: 0-> 1/16, 1-> 1/8
  byte  TempFlats, TempSharps     'one-measure accidentals
  byte  Scale, Note               'values to send to lcd
  byte  Acci                    'the sharp/flat bis corresponding to this note
  byte  Length                    'actual length of note
  byte  PrevLength, PrevScale     'dont send these unless different from prev 
OBJ
  pst      : "parallax serial terminal"
  lcd      : "lcdbare"
  
PUB main                                     'where to store cnt
    pst.Start (115_200)                        'start up ser terminal
    pst.str(String("hello, world   "))         'runs in cog 1
    pst.NewLine
    waitcnt(clkfreq/10+cnt)
    
    lcd.StartDisplay

    PlayTune

pub PlayTune               'come here to play a tune
    ParseHeader            'read and process header material
    PlayNotes              'play the actual tune
    pst.newline

PUB PlayNotes    
    repeat
      MyAscii := GetNextChar(MyPoint++)
      case MyAscii
        EOF:  Return      'end of file
        " ", CR:           'ignore end of line, space
        "|":              'end of measure
          TempFlats := 0
          TempSharps := 0   'turn off temporary accidentals
        $22:                'double quote
          repeat until MyAscii == $22
            MyAscii := GetNextChar(MyPoint++)   'skip over guitar chord  
        "A":
          Acci := ABit
          Note := An     'the note
          Scale := Scale5  'scale sans comma
          OctaveDown
          SendNewNote    
          
        "B":
          Acci := BBit
          Note := Bn     'the note
          Scale := Scale5  'scale sans comma
          OctaveDown
          SendNewNote   

        "C":
          Acci := CBit
          Note := Cn     'the note
          Scale := Scale4  'scale sans comma
          OctaveDown
          SendNewNote   

        "D":
          Acci := DBit
          Note := Dn     'the note
          Scale := Scale4  'scale sans comma
          OctaveDown
          SendNewNote 

        "E":
          Acci := EBit
          Note := En     'the note
          Scale := Scale4  'scale sans comma
          OctaveDown
          SendNewNote 

        "F":
          Acci := FBit
          Note := Fn     'the note
          Scale := Scale4  'scale sans comma
          OctaveDown
          SendNewNote 

        "G":
          Acci := GBit
          Note := Gn     'the note
          Scale := Scale4  'scale sans comma
          OctaveDown
          SendNewNote

        "a":
          Acci := ABit
          Note := An     'the note
          Scale := Scale6  'scale sans single quote
          OctaveUp
          SendNewNote           

        "b":
          Acci := BBit
          Note := Bn     'the note
          Scale := Scale6  'scale sans single quote
          OctaveUp
          SendNewNote                                              

        "c":
          Acci := CBit
          Note := Cn     'the note
          Scale := Scale5  'scale sans single quote
          OctaveUp
          SendNewNote  

        "d":
          Acci := DBit
          Note := Dn     'the note
          Scale := Scale5  'scale sans single quote
          OctaveUp
          SendNewNote

        "e":
          Acci := EBit
          Note := En     'the note
          Scale := Scale5  'scale sans single quote
          OctaveUp
          SendNewNote

        "f":
          Acci := FBit
          Note := Fn     'the note
          Scale := Scale5  'scale sans single quote
          OctaveUp
          SendNewNote

        "g":
          Acci := FBit
          Note := Gn     'the note
          Scale := Scale5  'scale sans single quote
          OctaveUp 
          SendNewNote

'*************see if scale down or up:  skip over comma or apost**********          
Pub OctaveDown
    if (GetNextChar(MyPoint) == ",")  'peak ahead
      Scale--         'up one scale                               
      MyPoint++       'skip over the comma
    FixLength
      
Pub OctaveUp
    if (GetNextChar(MyPoint) == "'")  'peak ahead
      Scale++         'up one scale                               
      MyPoint++       'skip over the apostrophe
    FixLength
      
'***************correct length if necessary*************
PUB FixLength
    Length := lookupz(LCode: Sixteenth, Eighth)   'basic
    MyAscii := GetNextChar(MyPoint)
    if ((MyAscii > "0") and (MyAscii < ":"))
      MyPoint++                    'skip over length modifier
      if (LCode == 0)             'basic is 1/16  
        case MyAscii
          "1": Length := Sixteenth
          "2": Length := Eighth
          "3": Length := Quarter
          "4": Length := Quarter
          "8": Length := Half

      elseif (LCode == 1)            'basic is 1/8
        case MyAscii
          "1": Length := Eighth
          "2": Length := Quarter
          "3": Length := Half
          "4": Length := Half
          "8": Length := Whole
                
    if ((Acci & Sharps) <> 0)
      Note++                   'up one-half step
    if ((Acci & Flats) <> 0)
      Note--                    'down one-half                           

'**********actually transmit the note to the LCD                        
Pub SendNewNote
     pst.newline
     if (scale <> prevscale)
       pst.str(string("   Scale: "))  'send new scale if needful
       pst.dec(scale)
       lcd.serialchar(scale)
       prevscale := scale

     if (Length <> prevlength)        'send new length code if needful
       pst.str(string("   Length Code: "))
       pst.dec(length)           
       lcd.serialchar(Length)
       prevLength := Length

     pst.str(string("   Note: "))
     pst.dec(Note)
     lcd.serialchar(note)
   ' waitcnt(clkfreq+cnt)
          

PUB ParseHeader
    MyPoint := 0           'move to front of file
    HaveKey := 0           'have not found the K: in the header
    repeat until (haveKey == 1)
       MyAscii := GetNextChar(MyPoint++)      'get the key       
       case MyAscii
         "X":              'sequence number
           repeat until myAscii  == CR
             MyAscii := GetNextChar(MyPoint++)
             pst.char(MyAscii) 
             
         "T":              'title
             
           lcd.serialChar(128)               'print title on top line
           repeat until myAscii  == CR
             MyAscii := GetNextChar(MyPoint++)
             pst.char(MyAscii)
             if ((MyAscii <> CR) & (MyAscii <> ":"))
                lcd.serialchar(MyAscii)    'put title on lcd
                
         "C":              'composer
           repeat until myAscii  == CR
             MyAscii := GetNextChar(MyPoint++)
             pst.char(MyAscii)

         "N":              'notes
           repeat until myAscii  == CR
             MyAscii := GetNextChar(MyPoint++)
             pst.char(MyAscii)

         "M":              'meter
           repeat until myAscii  == CR
             MyAscii := GetNextChar(MyPoint++)
             pst.char(MyAscii)

         "L":              'length
           GetNextChar(MyPoint++)    'skip over colon
           getNextChar(MyPoint++)    'skip over "1"
           GetNextChar(MyPoint++)    'skip over "/"
           MyAscii := GetNextChar(MyPoint++)   'get first digit
             if (MyAscii == 1)
               LCode := 0                     '1/16
               GetNextChar(MyPoint++)    'skip over assumed 6 as in 1/16
             else
               LCode := 1                     '1/8        
           pst.str(string("LengthCode: "))
           pst.dec(LCode)                                                              
           pst.newline
                          
         "K":              'header for key
           MyPoint++      'skip over colon           
           Key := GetNextChar(MyPoint++)   'get key (leaves mypoint at cr)
           pst.str(string("Found Key: "))
           pst.char(Key)
           haveKey := 1     
           case Key
             "C":
               flats := 0       'good ol' key of C
               sharps := 0      
             "G":
               Sharps := FBit
               Flats := 0
             "D":
               Sharps := FBit | CBit
               Flats := 0
             "F":
               Sharps := 0
               Flats := BBit               
             "B":  
               Sharps := 0
               Flats := BBit | EBit
           
             other:
               pst.str(string(" Never was very good at that key!"))
               repeat

    pst.str(string("  Flats: "))
    pst.bin(Flats,8)
    pst.str(string("  Sharps: "))
    pst.bin(Sharps,8)
    pst.str(string("  LCode: "))
    pst.dec(LCode)


PUB  GetNextChar(Pointer) 
     return byte[@Farewel + Pointer]             
dat
Happy   byte   "X: 3", CR
        byte   "T: Happy Birthday to You", CR
        byte   "C: Composer", CR
        byte   "N: Notes about the tune", CR
        byte   "M:3/4", CR
        byte   "L:1/8", CR
        byte   "K:G", CR
        byte   "D>D | E2 D2 G2 | F4 D>D | E2 D2 A2 | G4 D>D | d2 B2 G2 | (F2 E2) c>c |", CR
        byte   "B2 G2 A2 | G8|)", CR
        byte    EOF
        
CScale  byte   "X: 3", CR
        byte   "T: C Scales", CR
        byte   "C: Me", CR
        byte   "N: Four ocaves in the key of C", CR
        byte   "M:3/4", CR
        byte   "L:1/8", CR
        byte   "K:G", CR
        byte   "C, D, E, F, G, A, B, C D E F G A B c d e f g a b c' d' e' f' g' a' b'", CR
        byte   $FF

TwoNote byte   "X: 3", CR
        byte   "T: Short Song", CR
        byte   "C: Me", CR
        byte   "N: Used for testing", CR
        byte   "M:3/4", CR
        byte   "L:1/8", CR
        byte   "K:G", CR
        byte   "C D", CR
        byte   $FF           

Lengths byte   "X: 3", CR
        byte   "T: Lengths", CR
        byte   "C: Me", CR
        byte   "N: Play various note lengths", CR
        byte   "M:3/4", CR
        byte   "L:1/8", CR
        byte   "K:G", CR
        byte   "C C2 C4 C8", CR
        byte   $FF

Farewel byte   "X: 1", CR
        byte   "T: Ashokan Farewell", CR
        byte   "M:3/4", CR
        byte   "L:1/8", CR
        byte   "K:D", CR
        byte   "Ac|d3cBA|F4EF|G3FED|B,2D3B,|A,2D2F2|", CR
        byte   "A2d2f2|f3gf2|e4Ac|d3cBA|F4EF|G3FED|", CR
        byte   "B,2D3B,|A,2D2F2|A2d2f2|A2c2e2|d4",CR     
        byte    $FF                           